package FDN::Adsl::Factu;

use strict;
use Exporter;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		cloture_mois
		creation_factures
		creation_facture
		creation_prelevement
		envoyer_prelevement
		encaisser_facture
		encaisser_prelevement
		encaisser_prllig
		encaisser_prlligsepa
		encaisser_paiement
		changer_etat_paiement
		rejeter_prllig
		rejeter_prlligsepa
		jours
		);
our @EXPORT_OK = qw ();

use FDN::Common::Logger;
use FDN::Common::Constantes;
use FDN::Common::Modules;
use FDN::Etebac::Sepa;
use FDN::Adsl::Basic;
use POSIX qw(strftime);

our @jours =     ( 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
our @jours_bis = ( 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
our $debug = 0;
sub __debug($) {
	return if ! $debug;
	debug $_[0];
}

sub jours($) {
	my ( $annee ) = @_;
	if ($annee % 4) {
		# Pas divisible par 4
		return @jours;
	}
	# Divisible par 4
	if ($annee % 100) {
		# Mais pas divisible par 100
		return @jours_bis;
	}
	# Divisible par 100
	if ($annee % 400) {
		# Mais pas divisible par 400
		return @jours;
	}
	# Divisible par 400
	# note: Les regles a partir de 2000 ne sont pas encore officiellement en application
	return @jours_bis;
}


#sub facture_creance($$) {
#	my ( $fid, $creance ) = @_;
#	my $crid = $creance->{'CREANCE_ID'};
#	return FALSE if ( !defined $crid );
#	set_creance_facture($crid,$fid);
#	set_creance_etat($crid,'facturee');
#	return TRUE;
#}

sub cloture_mois($$) { # Test
	my ( $annee, $mois ) = @_;
	my $cloture = cloture_par_date($annee.$mois);
	if ( defined $cloture ) {
		if (!annuler_cloture($cloture)) {
			error "Pas moyen d'annuler la cloture precedente pour le moi $annee / $mois";
			return 0;
		}
	}
	my $clid = new_cloture($annee.$mois);
	if ( ! $clid ) {
		error "Je n'arrive pas a creer une cloture";
		return 0;
	}
	my @clients = tous_clients_actifs();
	return if ( array_has_no_rows(\@clients) );
	for my $cl ( @clients ) {
		cloture_mois_client($clid, $annee, $mois, $cl);
	}
	return $clid;
}

sub cloture_mois_client($$$$) { # Test
	my ( $clid, $annee, $mois, $client ) = @_;
	my $cid = $client->{'CLIENT_ID'};
	info 'Cloture du mois '.$annee.'/'.$mois.' pour le client '.$cid;
	my @ls = ligne_par_client_id($cid);
	for my $ligne ( @ls ) {
		cloture_mois_ligne($clid, $annee, $mois, $client, $ligne);
	}
	my @vs = vpn_par_client_id($cid);
	for my $vpn ( @vs ) {
		cloture_mois_vpn($clid, $annee, $mois, $client, $vpn);
	}
	my @ps = payant_par_client_id($cid);
	for my $payant ( @ps ) {
		cloture_mois_payant($clid, $annee, $mois, $client, $payant);
	}
	my @ds = don_par_client_id($cid);
	for my $don ( @ds ) {
		cloture_mois_don($clid, $annee, $mois, $client, $don);
	}
}

sub cloture_mois_vpn($$$$$) {
	my ( $clid, $annee, $mois, $client, $vpn ) = @_;
	my ( $ht, $ttc, $libelle );

	info 'Cloture du mois '.$annee.'/'.$mois.' pour le VPN id='.$vpn->{'VPN_ID'};
	my $prix = prix_par_id($vpn->{'PRIX_ID'});
	my $cid  = $client->{'CLIENT_ID'};
	my $tva  = $prix->{'PRIX_TVA'};

	my $prorata = vpn_prorata($vpn,$annee,$mois);
	if ( $prorata > 0 ) {
		$ttc = $prorata * $prix->{PRIX_ABOTTC};
		$ttc =~ s/\..*$//;
		$ht  = $prorata * $prix->{PRIX_ABOHT};
		$ht  =~ s/\..*$//;
		$libelle = "Abo VPN ".$vpn->{VPN_ID}." $annee/$mois";
		my $crid = new_creance($cid, $clid, $vpn->{'PRIX_ID'}, $ht, $ttc, $tva, $libelle);
		dater_creance($crid,$annee,$mois);
	}
}

sub cloture_mois_ligne($$$$$) { # Test
	my ( $clid, $annee, $mois, $client, $ligne ) = @_;
	my ( $ht, $ttc, $libelle );

	info 'Cloture du mois '.$annee.'/'.$mois.' pour la ligne '.$ligne->{'LIGNE_TEL'};
	my $prix = prix_par_id($ligne->{PRIX_ID});
	my $cid  = $client->{CLIENT_ID};
	my $tva  = $prix->{PRIX_TVA};
                
	if (if_fas($ligne,$annee,$mois)) {
		$ttc = $prix->{PRIX_FASTTC};
		$ht  = $prix->{PRIX_FASHT};
		$libelle = "FAS ".$ligne->{LIGNE_TEL};
		my $crid = new_creance($cid, $clid, $ligne->{PRIX_ID}, $ht, $ttc, $tva, $libelle);
		dater_creance($crid,$annee,$mois);
	}
	my $prorata=ligne_prorata($ligne,$annee,$mois);
	if ($prorata>0) {
		$ttc = $prorata * $prix->{PRIX_ABOTTC};
		$ttc =~ s/\..*$//;
		$ht  = $prorata * $prix->{PRIX_ABOHT};
		$ht  =~ s/\..*$//;
		$libelle = "Abo ".$ligne->{LIGNE_TEL}." $annee/$mois";
		my $crid = new_creance($cid, $clid, $ligne->{PRIX_ID}, $ht, $ttc, $tva, $libelle);
		dater_creance($crid,$annee,$mois);
	}
}

sub cloture_mois_payant($$$$$) { # Test
	my ( $clid, $annee, $mois, $client, $payant ) = @_;

	info 'Cloture du mois '.$annee.'/'.$mois.' pour le payant '.$payant->{'PAYANT_LIBELLE'};
	my $prix    = prix_par_id($payant->{'PRIX_ID'});
	my $cid     = $client->{'CLIENT_ID'};
	my $pid     = $payant->{'PAYANT_ID'};
	my $type    = $payant->{'PAYANT_TYPE'};
	my $date    = $payant->{'PAYANT_DATE'};
	my $debut   = $payant->{'PAYANT_DEBUT'};
	my $fin     = $payant->{'PAYANT_FIN'};
	my $libelle = $payant->{'PAYANT_LIBELLE'};
	my $tva     = $prix->{'PRIX_TVA'};
	my $fasht   = $prix->{'PRIX_FASHT'};
	my $fasttc  = $prix->{'PRIX_FASTTC'};
	my $aboht   = $prix->{'PRIX_ABOHT'};
	my $abottc  = $prix->{'PRIX_ABOTTC'};
	if ( $type !~ /^(unique|mensuel|annuel)$/ ) {
		error "J'essaye de facturer un type de PAYANT ($type) que je ne connais pas";
		return;
	}

	my $suffix;
	
	$suffix = {
		'unique' => '',
		'mensuel' => ' (FAS)',
		'annuel'  => ' (1ere annee)' };
	if ( $type eq 'unique' ) {
		$debut = $date;
	}

	if ( if_ce_mois_ci($debut,$annee,$mois) ) {
		my $crid = new_creance($cid, $clid, $payant->{'PRIX_ID'}, $fasht, $fasttc, $tva,
			$libelle.$suffix->{$type});
		dater_creance($crid,$annee,$mois);
	}

	if ( $type eq 'unique' ) {
		return;
	}
	
	$suffix = {
		'unique' => '',
		'mensuel' => '',
		'annuel'  => ' (annuel)' };
	if ( payant_abo($type,$debut,$fin,$annee,$mois) ) {
		my $crid = new_creance($cid, $clid, $payant->{'PRIX_ID'}, $aboht, $abottc, $tva,
			$libelle.$suffix->{$type});
		dater_creance($crid,$annee,$mois);
	}
}

sub cloture_mois_don($$$$$) { # FIXME NOTEST
	my ( $clid, $annee, $mois, $client, $don ) = @_;

	info 'Cloture du mois '.$annee.'/'.$mois.' pour le don '.$don->{'DON_LIBELLE'};
	my $type = $don->{'DON_TYPE'};
	if ( $type !~ /^(ponctuel|mensuel)$/ ) {
		error "J'essaye de facturer un type de DON ($type) que je ne connais pas";
		return;
	}

	if ( $don->{'DON_STATE'} ne 'ok' ) {
		return;
	}

	my ( $debut, $fin );
	if ( $type eq 'ponctuel' ) {
		$debut = $don->{'DON_DATE'};
		$fin   = '9999-12-31 00:00:00';
	} else {
		$debut = $don->{'DON_DEBUT'};
		$fin   = $don->{'DON_FIN'};
	}

	my $cid = $don->{'CLIENT_ID'};
	my $euros = $don->{'DON_MONTANT'};
	my $libelle = $don->{'DON_LIBELLE'};
	if ( payant_abo('mensuel',$debut,$fin,$annee,$mois) ) {
		my $crid = new_creance($cid, $clid, -1, $euros, $euros, '0', $libelle);
		dater_creance($crid, $annee, $mois);
	}
	if ( $type eq 'mensuel' && if_ce_mois_ci($fin,$annee,$mois) ) {
		set_don_state($don->{'DON_ID'}, 'ended');
	}
	if ( $type eq 'ponctuel' && if_ce_mois_ci($debut,$annee,$mois) ) {
		set_don_state($don->{'DON_ID'}, 'done');
	}
}

sub migrate_prelevemt($) {
	my ( $prl ) = @_;
	my $pid = $prl->{'PRELEVEMT_ID'};

	my $banque = $prl->{'PRELEVEMT_BANQUE'};
	my $guichet = $prl->{'PRELEVEMT_GUICHET'};
	my $compte = $prl->{'PRELEVEMT_COMPTE'};
	my $clef = $prl->{'PRELEVEMT_RIB'};

	my $bic  = rib_to_bic ($banque, $guichet, $compte, $clef);
	my $iban = rib_to_iban($banque, $guichet, $compte, $clef);
	if ( ! $bic || ! $iban ) {
		return undef;
	}

	set_prelevemt_bic($pid, $bic);
	set_prelevemt_iban($pid, $iban);
	set_prelevemt_mandat($pid, '++FDN-'.$prl->{'CLIENT_ID'}.'-'.$pid.'-MIGRATED');
	set_prelevemt_datemandat($pid, '2014-07-01');

}

sub dater_creance($$$) { # Test
	my ( $crid, $annee, $mois ) = @_;
	if ( $crid ) {
		set_creance_datedue($crid, $annee.$mois.(jours($annee))[$mois-1].'000000');
	} else {
		error "Je n'arrive pas a creer une creance!";
	}
}

sub annuler_cloture($) { # Test
	my ( $cloture ) = @_;
	my $clid = $cloture->{'CLOTURE_ID'};
	my @crs = creance_par_cloture_id($clid);
	for my $cr ( @crs ) {
		if ( ! annuler_creance($cr) ) {
			error "Pas moyen d'annuler la creance ".$cr->{'CREANCE_ID'};
			return 0;
		}
	}
	del_cloture($clid);
	return 1;
}

sub annuler_creance($) { # Test
	my ( $creance ) = @_;
	my $crid = $creance->{'CREANCE_ID'};
	if ( $creance->{'CREANCE_ETAT'} !~ /^(due|facturee)$/ ) {
		error "La creance $crid est dans un etat tel qu'elle ne peut pas etre annulee";
		return 0;
	}
	if ( $creance->{'CREANCE_ETAT'} eq 'facturee' ) {
		set_creance_facture($crid,-1);
	}
	set_creance_etat($crid,'annulee');
	set_creance_cloture($crid,-1);
	return 1;
}

sub creation_factures() { # Test
	my @cls = tous_clients_facture();
	my $nbfact = 0;
	for my $cl ( @cls ) {
		my $clid = $cl->{'CLIENT_ID'};
		my @crs = creance_par_client_id_etat($clid,'due');
		next if ( $#crs == -1 );
		my $fid = creation_facture($clid, \@crs);
		if ( $fid ) {
			$nbfact++;
		} else {
			error "Erreur a la creation d'une facture...";
		}
	}
	return $nbfact;
}

sub creation_facture($$) { # Test
	my ( $cid, $crs ) = @_;
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
	$mon++;
	$mon =~ s/^(.)$/0$1/;
	
	my $fnum = strftime "%Y%m%d-%H%M%S", localtime;
	$fnum .= '-'.$cid;
	my $fid = new_facture($cid,$fnum);
	for my $cr ( @$crs ) {
		my $crid = $cr->{'CREANCE_ID'};
		set_creance_facture($crid,$fid);
		set_creance_etat($crid,'facturee');
	}
	return $fid;
}

sub creation_prelevement($$$;$) { # Test
	my ( $nom, $fichier, $date, $prltype ) = @_;
	$prltype //= 'sepa';

	my $pid;

	my @cls = tous_clients_prelevement();
	for my $cl ( @cls ) {
		my $clid = $cl->{'CLIENT_ID'};
		my $prid = $cl->{'PRELEVEMT_ID'};
		my $prl  = prelevemt_par_id($prid);
		next if ( $prl->{'PRELEVEMT_ETAT'} ne 'ok' );
		if ( ! defined $prl->{'PRELEVEMT_BIC'} ) {
			migrate_prelevemt($prl);
			$prl = prelevemt_par_id($prid);
		}
		if ( $prltype eq 'sepa' && ! defined $prl->{'PRELEVEMT_BIC'} ) {
			error "Echec de migration du prelevemnt $prid pour le client $clid. J'ai du echouer a trouver son BIC.";
			next;
		}
		my $max;
		if ( defined $prl->{'PRELEVEMT_MAXPREL'} ) {
			$max = $prl->{'PRELEVEMT_MAXPREL'};
		} else {
			$max = 0;
		}

		my @crs = creance_par_client_id_etat($clid,'due');
		next if ( array_has_no_rows(\@crs) );
		my $total = 0;
		my $grand_total = 0;
		my @taken = ();
		@crs = sort { $a->{'CREANCE_TTC'} <=> $b->{'CREANCE_TTC'} } @crs;
		for my $cr ( @crs ) {
			my $crid = $cr->{'CREANCE_ID'};
			$grand_total += $cr->{'CREANCE_TTC'};
			if ( $max && $total + $cr->{'CREANCE_TTC'} > $max ) {
				debug "La creance $crid est ecartee, le total est de '$total', avec '$cr->{'CREANCE_TTC'}' en plus, ca fait plus que '$max'.";
				# Ca fait trop si on prend aussi cette creance la...
				next;
			}
			set_creance_etat($crid,'facturee');
			$total += $cr->{'CREANCE_TTC'};
			push @taken, $cr;
			debug "La creance $crid est prise, le total est maintenant de '$total'";
		}
		if ( @taken == 0 ) {
			# On n'a pas pu prendre la moindre creance...
			# Il faut remonter une erreur, sinon la situation restera definitivement bloquee...
			# FIXME Besoin d'une bonne idee.
			# die "Le client nous doit de l'argent qu'il ne pourra jamais payer a cause de MAXPREL.";
		} elsif ( $total == 0 ) {
			for my $cr ( @taken ) {
				my $crid = $cr->{'CREANCE_ID'};
				set_creance_etat($crid,'payee');
			}
		} elsif ( $total > 0 ) {
			if ( ! defined $pid ) {
				$pid = new_prlbq($nom,$fichier,$date,$prltype);
			}
			if ( $prltype eq 'etebac' ) {
				my $ligid = new_prllig($pid,
					$clid,
					$prl->{'PRELEVEMT_BANQUE'},
					$prl->{'PRELEVEMT_GUICHET'},
					$prl->{'PRELEVEMT_COMPTE'},
					$prl->{'PRELEVEMT_DOMICIL'},
					$prl->{'PRELEVEMT_TITULAIRE'},
					$total,
					$clid.' - '.$nom
				);
				for my $cr ( @taken ) {
					my $crid = $cr->{'CREANCE_ID'};
					set_creance_prllig($crid,$ligid);
				}
			}
			if ( $prltype eq 'sepa' ) {
				my $ligid = new_prlligsepa($pid,
					$prl->{'PRELEVEMT_ID'},
					$clid,
					$prl->{'PRELEVEMT_BIC'},
					$prl->{'PRELEVEMT_IBAN'},
					$prl->{'PRELEVEMT_MANDAT'},
					$prl->{'PRELEVEMT_DATEMANDAT'},
					$prl->{'PRELEVEMT_TITULAIRE'},
					$total,
					$clid.' - '.$nom
				);
				if ( $prl->{'PRELEVEMT_FIRST'} ) {
					set_prlligsepa_first($ligid,1);
					set_prelevemt_first($prid,0)
				}
				for my $cr ( @taken ) {
					my $crid = $cr->{'CREANCE_ID'};
					set_creance_prlligsepa($crid, $ligid);
				}
			}
		} else {
			debug "On doit encore de l'argent a ce type-la: $clid";
			for my $cr ( @taken ) {
				my $crid = $cr->{'CREANCE_ID'};
				set_creance_etat($crid, 'due');
			}
			if ( $grand_total > 0 ) {
				# FIXME Remonter la meme erreur que ci-dessus, bien que le cas soit plus tordu.
				# die "Il nous doit de l'argent qu'il ne pourra jamais payer à cause de MAXPREL.";
			}
		}
	}
	return $pid;
}

sub envoyer_prelevement_etebac($$) { # Test
	my ( $pid, $bqdst ) = @_;
	my $pr = prlbq_par_id($pid);
	my @ls = prllig_par_prlbq_id($pid);

	conduse('FDN::Common::Config');
	my $cfg = Config_Section('adsl/global.cfg',"banque-$bqdst");
	if ( ! defined $cfg ||
	     ! defined $cfg->{'emetteur'} ||
	     ! defined $cfg->{'raison-soc'} ||
	     ! defined $cfg->{'banque'} ||
	     ! defined $cfg->{'guichet'} ||
	     ! defined $cfg->{'compte'} ||
	     ! defined $cfg->{'domicil'} ) {
		return "Cette banque est mal definie...";
		# Panic
	}

	conduse('FDN::Etebac::Etebac3');
	my $eth = open_prelevement({
		'emetteur'   => $cfg->{'emetteur'},
		'date'       => $pr->{'PRLBQ_DATE'},
		'raison-soc' => $cfg->{'raison-soc'},
		'dst' => {
			'banque'  => $cfg->{'banque'},
			'guichet' => $cfg->{'guichet'},,
			'compte'  => $cfg->{'compte'},
			'domiciliation' => $cfg->{'domicil'}
		}
	});
	for my $l ( @ls ) {
		print_prelevement($eth, {
			'nom' => $l->{'PRLLIG_TITULAIRE'}.'',
			'src' => {
				'banque' => $l->{'PRLLIG_BANQUE'},
				'guichet' => $l->{'PRLLIG_GUICHET'},
				'compte' => $l->{'PRLLIG_COMPTE'},
				'domiciliation' => $l->{'PRLLIG_DOMICIL'}
			},
			'libelle' => $l->{'PRLLIG_LIBELLE'},
			'montant' => $l->{'PRLLIG_MONTANT'},
			'emetteur' => $cfg->{'emetteur'}
		});
	}
	my $res = close_prelevement($eth,{
		'emetteur' => $cfg->{'emetteur'}
	});

	conduse('FDN::Common::Config');
	my $dir = Config_Key('adsl/global.cfg','dirs','prldir');
	$dir = '/tmp' if ! defined $dir;
	my $file = $pr->{'PRLBQ_FICHIER'};
	$file = 'test.txt' if ! defined $file;
	if ( ! open OUTPUT, ">$dir/$file" ) {
		error "Failed to open file $dir/$file to write prl.";
		return undef;
	}
	print OUTPUT $res;
	close OUTPUT;
	set_prlbq_etat($pid,'envoye');
	return $res;
}

sub envoyer_prelevement_sepa($$) {
	my ( $pid, $bqdst ) = @_;
	my $pr = prlbq_par_id($pid);
	my @ls = prlligsepa_par_prlbq_id($pid);

	conduse('FDN::Common::Config');
	my $dir = Config_Key('adsl/global.cfg','dirs','prldir');
	$dir = '/tmp' if ! defined $dir;
	my $filename = $pr->{'PRLBQ_FICHIER'};
	$filename = 'test.txt' if ! defined $filename;
	$filename = $dir.'/'.$filename;


	my $dd = dd_init_file($bqdst, $pr->{'PRLBQ_DATE'});
	for my $l ( @ls ) {
		my $type = 'RCUR';
		if ( $l->{'PRLLIGSEPA_FIRST'} ) {
			$type = 'FRST';
		}
		dd_add_line($dd, $type, $l);
	}
	dd_make_file($dd, $filename);
	set_prlbq_etat($pid,'envoye');

	local $/;
	open FILE, $filename;
	my $res = <FILE>;
	close FILE;
	return $res;
}

sub envoyer_prelevement($$) { # Test
	my ( $pid, $bqdst ) = @_;
	my $pr = prlbq_par_id($pid);
	if ( $pr->{'PRLBQ_TYPE'} eq 'sepa' ) {
		return envoyer_prelevement_sepa($pid, $bqdst);
	} else {
		return envoyer_prelevement_etebac($pid, $bqdst);
	}
}

sub encaisser_facture($) { # Test
	my ( $fid ) = @_;
	my $facture = facture_par_id($fid);
	my @crs = creance_par_facture_id($fid);
	my $ttc = 0;
	my $ht  = 0;
	for my $cr ( @crs ) {
		my $crid = $cr->{'CREANCE_ID'};
		set_creance_etat($crid,'payee');
		set_creance_datepayee($crid,'now');
		$ttc += $cr->{'CREANCE_TTC'};
		$ht  += $cr->{'CREANCE_HT'};
	}
	set_facture_etat($fid,'payee');
	set_facture_payee($fid,'now');
	if ( $ttc || $ht ) {
		new_tva($ttc-$ht,"Reglement facture ".$facture->{'FACTURE_NUM'});
	}
}

sub encaisser_prelevement($$$) { # Test
	my ( $pid, $payes, $rejets ) = @_;
	my $prlbq = prlbq_par_id($pid);
	my $ht  = 0;
	my $ttc = 0;
	for my $pl ( @$rejets ) {
		__changer_prllig($pl,'rejet','due');
	}
	for my $pl ( @$payes ) {
		my ( $h, $t ) = __changer_prllig($pl,'paye','payee');
		$ht += $h;
		$ttc += $t;
	}
	set_prlbq_etat($pid,'paye');
	new_tva($ttc-$ht, 'Prelevement '.$prlbq->{'PRLBQ_NOM'});
}

sub encaisser_paiement($$$$$) { # Test
	my ( $cid, $lib, $num, $ttc, $type ) = @_;
	my $pid = new_paiement($cid,$lib,$type,$num,$ttc);
	my $crid;
	my $tid;
	if ( $pid ) {
		$crid = new_creance($cid,-1,-1,-$ttc,-$ttc,'0',"Paiement num. $pid");
	}
	return $pid;
}

sub changer_etat_paiement($$) { # Test
	my ( $pid, $etat ) = @_;
	my $p = paiement_par_id($pid);
	my $ttc = $p->{'PAIEMENT_TTC'};
	my $cid = $p->{'CLIENT_ID'};
	my $num = $p->{'PAIEMENT_NUM'};
	my $prev_etat = $p->{'PAIEMENT_ETAT'};
	if ( $prev_etat eq $etat ) {
		return undef;
	}
	my $crid;
	if ( $etat eq 'ko' ) {
		$crid = new_creance($cid,-1,-1,$ttc,$ttc,'0',"Rejet paiement num. $pid");
		set_paiement_etat($pid,'ko');
	} elsif ( $etat eq 'ok' ) {
		$crid = new_creance($cid,-1,-1,-$ttc,-$ttc,'0',"Reprise paiement num. $pid");
		set_paiement_etat($pid,'ok');
	} else {
		error "Le nouvel état n'est pas valide";
		return undef;
	}
	return $crid;
}

sub rejeter_prlligsepa($) { # Test
	my ( $pl ) = @_;
	my ( $ht, $ttc ) = __changer_prlligsepa($pl,'rejet','due');
	new_tva($ht-$ttc,"Rej. prel. ".$pl->{'PRLLIGSEPA_LIBELLE'});
}

sub encaisser_prlligsepa($) { # Test
	my ( $pl ) = @_;
	my ( $ht, $ttc ) = __changer_prlligsepa($pl,'paye','payee');
	new_tva($ttc-$ht,"Prel. ligne ".$pl->{'PRLLIGSEPA_LIBELLE'});
}

sub __changer_prlligsepa($$$) { # Test
	my ( $pl, $etat, $cretat ) = @_;
	__debug "Demande de changement de l'etat de la ligne ".$pl->{'PRLLIGSEPA_ID'}." pour passer a $cretat.";

	if ( $pl->{'PRLLIGSEPA_ETAT'} eq $etat ) {
		__debug "Deja dans cet etat.";
		return ( 0, 0);
	}
	my $ht  = 0;
	my $ttc = 0;
	my $plid = $pl->{'PRLLIGSEPA_ID'};
	my @crs = creance_par_prlligsepa_id($plid);
	if ( $#crs == -1 ) {
		__debug "Pas de creance associee a la ligne de prelevement $plid.";
		return ( 0, 0 );
	}
	for my $cr ( @crs ) {
		my $crid = $cr->{'CREANCE_ID'};
		__debug "Changement de la creance $crid.";
		set_creance_etat($crid,$cretat);
		$ttc += $cr->{'CREANCE_TTC'};
		$ht  += $cr->{'CREANCE_HT'};
	}
	__debug "Changement de la ligne $plid.";
	set_prlligsepa_etat($plid,$etat);
	if ( $etat eq 'rejet' && $pl->{'PRLLIGSEPA_FIRST'} ) {
		set_prelevemt_first($pl->{'PRELEVEMT_ID'}, 1);
	}
	return ( $ht, $ttc );
}

sub rejeter_prllig($) { # Test
	my ( $pl ) = @_;
	my ( $ht, $ttc ) = __changer_prllig($pl,'rejet','due');
	new_tva($ht-$ttc,"Rej. prel. ".$pl->{'PRLLIG_LIBELLE'});
}

sub encaisser_prllig($) { # Test
	my ( $pl ) = @_;
	my ( $ht, $ttc ) = __changer_prllig($pl,'paye','payee');
	new_tva($ttc-$ht,"Prel. ligne ".$pl->{'PRLLIG_LIBELLE'});
}

sub __changer_prllig($$$) { # Test
	my ( $pl, $etat, $cretat ) = @_;
	my $name;
	if ( exists $pl->{'PRLLIG_ID'} ) {
		$name = 'PRLLIG';
	} else {
		$name = 'PRLLIGSEPA';
	}
	__debug "Demande de changement de l'etat de la ligne ".$pl->{$name.'_ID'}." pour passer a $cretat.";

	if ( $pl->{$name.'_ETAT'} eq $etat ) {
		__debug "Deja dans cet etat.";
		return ( 0, 0);
	}
	my $ht  = 0;
	my $ttc = 0;
	my $plid = $pl->{$name.'_ID'};
	my @crs;
	if ( $name eq 'PRLLIG' ) {
		@crs = creance_par_prllig_id($plid);
	} else {
		@crs = creance_par_prlligsepa_id($plid);
	}
	if ( $#crs == -1 ) {
		__debug "Pas de creance associee a la ligne de prelevement $plid.";
		return ( 0, 0 );
	}
	for my $cr ( @crs ) {
		my $crid = $cr->{'CREANCE_ID'};
		__debug "Changement de la creance $crid.";
		set_creance_etat($crid,$cretat);
		$ttc += $cr->{'CREANCE_TTC'};
		$ht  += $cr->{'CREANCE_HT'};
	}
	__debug "Changement de la ligne $plid.";
	if ( $name eq 'PRLLIG' ) {
		set_prllig_etat($plid,$etat);
	} else {
		set_prlligsepa_etat($plid,$etat);
	}
	return ( $ht, $ttc );
}

sub if_ce_mois_ci($$$) { # Test
	my ( $date, $annee, $mois ) = @_;
	my $debut = $annee.'-'.$mois.'-01 00:00:00';
	my $fin   = $annee.'-'.$mois.'-31 00:00:00';
	if ( $date ge $debut && $date lt $fin ) {
		info 'Echeance ('.$date.') dans le mois (de '.$debut.' a '.$fin.')';
		return 1;
	} else {
		info 'Echeance ('.$date.') pas dans le mois (de '.$debut.' a '.$fin.')';
		return 0;
	}
}	       

sub if_fas($$$) { # Test
	my ( $ligne, $annee, $mois ) = @_;
	my $dtin = $ligne->{'LIGNE_DTFAC'};
	return if_ce_mois_ci($dtin, $annee, $mois);
}

#sub mois_prochain($) {
#	my ( $date ) = @_;
#	$date =~ /^(\d{4})-(\d{2})-(\d{2}) (\d\d):(\d\d):(\d\d)$/;
#	my ( $annee, $mois, $jour, $heure ) = ( $1, $2, $3, $4.$5.$6 );
#	$mois++;
#	if ( $mois == 13 ) {
#		$mois = 1;
#		$annee++;
#	}
#	while ( $jour >= (jours($annee))[$mois-1] ) {
#		$jour--;
#	}
#	$mois =~ s/^(.)$/0$1/;
#	$jour =~ s/^(.)$/0$1/;
#	return $annee.$mois.$jour.$heure;
#}
#
#sub annee_prochaine($) {
#	my ( $date ) = @_;
#	$date =~ /^(\d{4})-(\d{2})-(\d{2}) (\d\d):(\d\d):(\d\d)$/;
#	my ( $annee, $mois, $jour, $heure ) = ( $1, $2, $3, $4.$5.$6 );
#	$annee++;
#	return $annee.$mois.$jour.$heure;
#}

sub ligne_prorata($$$) { # Test
	my ( $ligne, $annee, $mois ) = @_;
	my $dtin = $ligne->{'LIGNE_DTFAC'};
	my $dtout = $ligne->{'LIGNE_DTOUT'};
	return __prorata($dtin,$dtout,$annee,$mois);
}

sub vpn_prorata($$$) {
	my ( $vpn, $annee, $mois ) = @_;
	my $dtin = $vpn->{'VPN_DTFAC'};
	my $dtout = $vpn->{'VPN_DTOUT'};
	return __prorata($dtin,$dtout,$annee,$mois);
}

sub payant_abo($$$$$) { # Test
	my ($type,$debut,$fin,$annee,$mois) = @_;
	my $pr = __prorata($debut,$fin,$annee,$mois);
	if ( ! $pr ) {
		return FALSE;
	}
	if ( $type eq 'mensuel' ) {
		return TRUE;
	}
	$debut =~ /^(\d{4})-(\d{2})-/;
	my ( $anneed, $moisd ) = ($1,$2);
	if ( $moisd == $mois ) {
		return TRUE;
	}
}
#sub payant_prorata($$$) { # Test
#	my ( $debut, $fin, $annee, $mois ) = @_;
#	my $pr = __prorata($debut,$fin,$annee, $mois);
#	if ( $pr > .5 ) {
#		return 1;
#	} else {
#		return 0;
#	}
#}

sub __prorata($$$$) { # Test
	my ( $dtin, $dtout, $annee, $mois ) = @_;

	my $njours = (jours($annee))[$mois-1];
	my $debut  = $annee.'-'.$mois.'-01 00:00:00';
	my $fin    = $annee.'-'.$mois.'-31 00:00:00';

	if ( $dtin  eq '0000-00-00 00:00:00' ) {
		info "Prorata mois du $debut au $fin, debut non defini: 0 (pas encore a priori)";
		return 0;
	}
	if ( $dtout eq '0000-00-00 00:00:00' ) {
		$dtout = '2999-12-31 00:00:00';
	}
	if ( $dtin lt $debut && $dtout gt $fin ) {
		info "Prorata mois du $debut au $fin, ligne du $dtin au $dtout: 1 (plein)";
		return 1;
	}
	if ( $dtout lt $debut ) {
		info "Prorata mois du $debut au $fin, ligne du $dtin au $dtout: 0 (resilie)";
		return 0;
	}
	if ( $dtin gt $fin ) {
		info "Prorata mois du $debut au $fin, ligne du $dtin au $dtout: 0 (pas encore)";
		return 0;
	}
	my $ijour=0;
	my $ojour=$njours;															       
	if ( $dtin gt $debut && $dtout lt $fin ) {
		$dtin =~ /^\d{4}-\d{2}-(\d{2}) \d\d:\d\d:\d\d$/;
		$ijour = $1 - 1;
		$dtout =~ /^\d{4}-\d{2}-(\d{2}) \d\d:\d\d:\d\d$/;
		$ojour = $1;
	}
	if ( $dtin gt $debut ) {
		$dtin =~ /^\d{4}-\d{2}-(\d{2}) \d\d:\d\d:\d\d$/;
		$ijour = $1 - 1;
	}
	if ( $dtout lt $fin ) {
		$dtout =~ /^\d{4}-\d{2}-(\d{2}) \d\d:\d\d:\d\d$/;
		$ojour = $1;
	}
	my $prorata = ($ojour-$ijour)/$njours;
	info "Prorata mois du $debut au $fin, ligne du $dtin au $dtout: $prorata (partiel)";
	return $prorata;
}

1;
