package FDN::Adsl::Plutar;

use strict;
use Exporter;
our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
	test_eligibilite
	get_pgp
);
our @EXPORT_OK = qw ();
our %EXPORT_TAGS = ();

use strict;
use FDN::Common::Modules;
use FDN::Common::Logger;
use FDN::Sql::DBC;
use FDN::Sql::Plutar::Constantes;
use FDN::Adsl::Basic;
use FDN::Adsl::SelN;
use FDN::Adsl::Wf;

our $dbh;
our $notify_str = '';

BEGIN {
	$dbh = do_connect('adsl');
}

sub get_dbh() {
	return $dbh;
}

sub get_notify() {
	return $notify_str;
}

sub test_eligibilite($) {
	my ( $args ) = @_;
#	conduse('FDN::Adsl::Sel9');
	conduse('FDN::Sql::Workflow::Run');
	workflow_select_db('adsl');
	my $offre = $args->{'offre'};
	$offre =~ s/option/opt/;
	my $tel   = $args->{'tel'};
	my $cp    = $args->{'cp'};
	my $eid   = $args->{'eid'};
	debug "J'interroge le serveur d'eligibilite sur l'offre $offre pour la ligne $tel";
	my $res;
	if ($offre =~ /Liazo/i) {
		conduse('FDN::Adsl::SelL');
		$res = FDN::Adsl::SelL::eligibilite($tel,$cp,[$offre]);
	} else {
		conduse('FDN::Adsl::SelN');
		$res = FDN::Adsl::SelN::eligibilite($tel,$cp,[$offre]);
	}
	if ( ! defined $args->{'resultats' } ) {
		$args->{'resultats'} = [];
	}
	if ( ! defined $res->{'body'} ) {
		$res->{'body'} = '<UNDEF>';
	}
	if ( $res->{'etat'} == 1 ) {
		# Serveur indisponible
		push @{$args->{'resultats'}}, "Retour etat 1 (serveur FT ou TD indispo). Retry.";
		return PLUTAR_RETRY;
	}
	if ( $res->{'etat'} == 2 ) {
		# Erreur chez FDN
		push @{$args->{'resultats'}}, "Retour etat 2. Bug FDN. A creuser.";
		set_eligibilite_reponse($eid,$res->{'body'});
		set_eligibilite_dttest($eid);
		send_event_elig($eid,0);
		return PLUTAR_FAILED;
	}
	if ( $res->{'etat'} == 3 ) {
		# Probleme sur le numero de telephone
		push @{$args->{'resultats'}}, "Retour etat 3. Numero de telephone invalide.";
		set_eligibilite_reponse($eid,$res->{'body'});
		set_eligibilite_dttest($eid);
		send_event_elig($eid,0);
		return PLUTAR_FAILED;
	}
	if ( $res->{'etat'} ) {
		# Etat impossible.
		push @{$args->{'resultats'}}, "Retour etat inconnu. Regarder la reponse dans la base.";
		set_eligibilite_reponse($eid,$res->{'body'});
		set_eligibilite_dttest($eid);
		send_event_elig($eid,0);
		return PLUTAR_FAILED;
	}
	if ( ! defined $res->{'offres'} ||
	     ! defined $res->{'offres'}->{$offre} ||
	     ! defined $res->{'offres'}->{$offre}->{'faisable'} ) {
	     	push @{$args->{'resultats'}}, "Reponse du module FDN::Adsl::SelN invalide.";
		set_eligibilite_reponse($eid,$res->{'body'});
		set_eligibilite_dttest($eid);
		send_event_elig($eid,0);
		return PLUTAR_FAILED;
	}
	set_eligibilite_reponse($eid,$res->{'body'});
	set_eligibilite_dttest($eid);
	if ( $res->{'offres'}->{$offre}->{'faisable'} ) {
		send_event_elig($eid,1);
		# send_event faisable
	} else {
		# send_event pas faisable
		send_event_elig($eid,0);
	}
	return PLUTAR_SUCCESS;
}

sub send_event_elig($$) {
	my ( $eid, $faisable ) = @_;
	send_event('ELIG-'.$eid,{'faisable'=>$faisable});
}

sub get_pgp($) {
	my ( $args ) = @_;
	my $pgp = $args->{'pgp'};
	my $email = $args->{'email'};
	my $res = system("gpg --keyserver hkp://pgp.mit.edu --recv-key $pgp > /dev/null 2>&1");
	if ($res == 0) {
		envoie_mail($email, 'pgp-success', $args);
		return PLUTAR_SUCCESS;
	} else {
		envoie_mail($email, 'pgp-failed', $args);
		return PLUTAR_RETRY;
	}
}

sub resil_change_radius($) {
	my ( $args ) = @_;
	my $rid = $args->{'radid'};
	my $email = $args->{'email'};

	my $ru = raduser_par_id($rid);
	if ( ! defined $ru ) {
		push @{$args->{'resultats'}}, "Le compte radius numero $rid n'existe pas.";
		return PLUTAR_FAILED;
	}
	my $login = $ru->{'RADUSER_LOGIN'};
	$args->{'ancien_login'} = $login;
	if ( $login =~ /fdn\.(fr|nerim|dslnet.fr)$/ ) {
		$login =~ s/fdn\.(fr|nerim|dslnet.fr)$/\/VALID/;
	} elsif ( $login =~ /VALID$/ ) {
		push @{$args->{'resultats'}}, "Le compte radius numero $rid est deja invalide";
		return PLUTAR_SUCCESS;
	} else {
		push @{$args->{'resultats'}}, "Le compte radius numero $rid ne se termine pas par 'fdn.fr' ou 'fdn.nerim' ou 'fdn.dslnet.fr'";
		return PLUTAR_FAILED;
	}
	set_raduser_login($rid,$login);
	envoie_mail($email, 'resil-change-rad', $args);
	return PLUTAR_SUCCESS;
}

1;
