package FDN::Adsl::SelL;

use strict;
use warnings;

use base 'Exporter';
our @ISA = ( 'Exporter' );
our @EXPORT = qw(
	eligibilite

	sign
	verify

	get_ville
	get_rue
	get_nd
	get_info_nd
	get_fiber
	get_sdsl
	get_adsl
	get_contracts
	get_contract
	get_services
	get_collecte
	commande_adsl
	get_commandes
	get_info_commande

	get_reponse_post
	get_reponse_get

	poll
	);
our @EXPORT_OK = qw(
	);

use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Web::TemplateParse;
use FDN::Sql::Basic;
use FDN::Sql::DB_Access;
use FDN::Adsl::OrdreTd;
use FDN::Adsl::DBC;
use FDN::Adsl::Basic;

use HTTP::Headers;
use HTTP::Request;
use LWP::UserAgent;
use JSON qw( decode_json encode_json );
use Encode qw ( encode );

use Digest::SHA;
use IPC::Open2;

our $debug = 1;

# Codes de faisabilit de Liazo:
# 0: En cours de raccordement
# 1: Possible
# 2: Impossible (voir raison)
# 3: Possible sous rserve technique
# 4: Possible sous condition commerciale

my $tbls = 'adsl/tables.cfg';
my $conf = 'adsl/elig/liazo.cfg';

sub __debug($) {
	my ( $msg ) = @_;
	debug $msg if ( $debug );
}

sub doJSONbool($) {
	my ($b) = @_;
	if ($b eq "true") {
		return JSON::true;
	} else {
		return JSON::false;
	}
}

# Signer un message  poster  Liazo
sub sign($) {
	#  gnrer avec
	# openssl ecparam -genkey -name prime256v1 -out private.pem
	# et envoyer la partie publique obtenue avec
	# openssl ec -in private.pem -pubout -out public.pem
	my $private_file = Config_Key($conf,'ariane','sign_key');
        my ($msg) = @_;
        my $digest = Digest::SHA::sha256($msg);
        my $signature;

        # Idalement
        #use Crypt::OpenSSL::ECDSA;
        #open FILE, $private_file or die "cle non lisible";
        #my $private;
        #{
        #       local $/ = undef;
        #       $private = <FILE>;
        #}
        #close FILE;
        #
        #$signature=Crypt::OpenSSL::ECDSA->sign($digest,$private);

        # Mais les modules perl ne sont pas encore l, donc on utilise le module python
        my ( $in, $out );
        my $pid = open2($out, $in, 'python -c "'.
                "import ecdsa;".
                "import sys;".

                "key=open('$private_file').read();".
                "sk=ecdsa.SigningKey.from_pem(key);".
                "sys.stdout.write(sk.sign(sys.stdin.read()));".
                '"');
        print $in $digest;
        close $in;
        waitpid($pid, 0);
        my $status = $? >> 8;
        die if $status != 0;
        {
                local $/ = undef;
                $signature = <$out>;
        }

	return unpack('H*', $signature);
}

# Vrifier une signature de message  poster  Liazo
sub verify($$) {
        my $public_file = Config_Key($conf,'ariane','sign_pub');
        my ($msg, $signature) = @_;
        my $digest = Digest::SHA::sha256($msg);

        # Idalement
        #use Crypt::OpenSSL::ECDSA;
        #open FILE, $public_file or die "cle non lisible";
        #my $public;
        #{
        #       local $/ = undef;
        #       $public = <FILE>;
        #}
        #close FILE;

        # Mais les modules perl ne sont pas encore l, donc on utilise le module python
        my ( $in, $out );
        my $pid = open2($out, $in, 'python -c "'.
                "import ecdsa;".
                "import sys;".
                "import binascii;".

                "signature='$signature';".
                "signature=binascii.unhexlify(signature);".

                "key=open('$public_file').read();".
                "vk=ecdsa.VerifyingKey.from_pem(key);".
                "vk.verify(signature,sys.stdin.read());".
                '"');
        print $in $digest;
        close $in;
        waitpid($pid, 0);
        my $status = $? >> 8;
        die if $status != 0;
}

sub get_eligibilite($) {
	my ($tel) = @_;
	my @elig_list = (("false", "false"), ("true", "false"), ("false", "true"));

	find_elig:
	while ($#elig_list > 0) {
		my $inactive = shift(@elig_list);
		my $precab = shift(@elig_list);

		my $info = get_adsl($tel,$inactive,$precab);
		if ( ! defined $info || ! defined $info->[0] ) {
			debug('Erreur serveur Liazo');
			return (undef, undef, "Erreur serveur Liazo", undef);
		}

		if ( $info->[0]->{'request'}->{'phone_number'} != $tel+0 ) {
			debug('Rponse ne portant pas sur le bon NDI. Bug?');
			return (undef, undef, "La rponse ne porte pas sur le bon numro de tlphone...", undef);
		}

		for my $offre ( @$info ) {
			if ($offre->{'response'}->{'feasibility_res'} eq "ND inconnu") {
				next;
			}
			if ($offre->{'response'}->{'feasibility_res'} eq "ND non attribu") {
				next;
			}
			debug("au moins la rponse ".$offre->{'response'}->{'feasibility_res'}." pour inactive:$inactive precab:$precab");
			return $inactive, $precab, undef, @$info;
		}
	}
	return (undef, undef, "Numro de tlphone inconnu", undef);
}

# Tester l'ligibilit d'un numro de tlphone
sub eligibilite($$$) {
	my ( $tel, $zip, $offres ) = @_;
	my $gres = {};
	$gres->{'offres'} = {};
	$gres->{'etat'} = 0;
	$gres->{'body'} = '';
	my $reverse = {};
	for my $offre ( @$offres ) {
		my $debit = get_offre($offre);
		next if ( ! defined $debit );
		$reverse->{$debit} = $offre;
	}

	my ( $inactive, $precab, $error, @infos ) = get_eligibilite($tel);

	if (defined($error)) {
		$gres->{'etat'} = 1;
		$gres->{'message'} = [ $error ];
		return $gres;
	}

	my $nd_info = get_info_nd($tel,$inactive,$precab);
	$gres->{'body'} = "Adresse ligne selon Liazo:\n".
		$nd_info->{'info'}{'street_nb'}." ".
		$nd_info->{'info'}{'street'}."\n".
		$nd_info->{'info'}{'city'};
	
	for my $offre ( @infos ) {
		# On ne s'embte pas avec du bi-vc pour l'instant.
		if ($offre->{'product'}->{'vc_type'} eq 'bi-vc') {
			next;
		}
		my $name = $offre->{'product'}->{'bandwidth_mb'};
		if ($offre->{'product'}->{'tech'} eq 'RE') {
			$name .= 'RE';
		}
		# Pour l'instant on n'a que cette option
		$name .= "LiazoOrange";
		if ($offre->{'product'}->{'dsl_only'}) {
			$name .= 'Nu';
		}
		__debug "examen offre $name";
		if (defined $reverse->{$name}) {
			__debug "reverse ".$reverse->{$name};
		}

		my $reason = '';
		if (scalar @{$offre->{'response'}->{'reason'}}) {
			for my $one_reason ( @{$offre->{'response'}->{'reason'}} ) {
				$reason .= $one_reason;
			}
		} else {
			$reason = $offre->{'response'}->{'feasibility_res'};
			if ($reason eq "ND non attribu") {
				$reason = "Numro de tlphone actuellement pas attribu";
			}
		}
		$reason =~ s/\x{2019}/'/;

		if ($offre->{'feasibility'} eq '1') {
			$offre->{'faisable'} = 1;
		} elsif ($offre->{'feasibility'} eq '3') {
			$offre->{'faisable'} = 1;
			$offre->{'reserve'} = $reason;
		} elsif ($offre->{'feasibility'} eq '4') {
			$offre->{'faisable'} = 1;
			$offre->{'reserve'} = $reason;
		} else {
			$offre->{'faisable'} = 0;
		}
		$offre->{'LineLength'} = $offre->{'length'};
		$offre->{'Attenuation'} = '';
		__debug "faisable: ".$offre->{'faisable'};
		if ($offre->{'faisable'} eq 0) {
			$offre->{'infos'} = '';
			$offre->{'infos'} .= $reason;
			__debug "pas faisable: ".$offre->{'infos'};
		}
		if (defined $reverse->{$name}) {
			$offre->{'offre'} = $reverse->{$name};
			$gres->{'offres'}->{$reverse->{$name}} = $offre;
		}
	}

	return $gres;
}

# Usage interne: convertit un code d'offre FDN en notre propre code
sub get_offre($) {
	my ( $offre ) = @_;
	my ( $debit, $option );
	if ( $offre =~ /^(\d+(k|M|RE))(LiazoOrange(|Nu))$/ ) {
		( $debit, $option ) = ($1,$3);
	} else {
		__debug "Offre inconnue (syntaxe): $offre";
		return undef;
	}
	my $res; 
	$res = Config_Key($tbls,"$option-liazo",$debit);
	if ( ! defined $res ) {
		__debug "Offre inconnue (config): $offre, $debit, $option";
		return undef;
	}
	$res .= $option;
	__debug "Offre trouvee, $offre se dit $res.";
	return $res; 
}

# Demandes d'informations
sub get_ville($;$) {
	my ( $query, $dept ) = @_;
	$query = "query=".$query;
	if (defined $dept) {
		$query = $query."&depart_code=$dept";
	}
	return get_reponse_get('/geobase/list/city', $query);
}

sub get_rue($;$$) {
	my ( $query, $ville, $zip ) = @_;
	$query = "query=".$query;
	if (defined $ville) {
		$query = $query."&city=$ville";
	}
	if (defined $zip) {
		$query = $query."&post_code=$zip";
	}
	return get_reponse_get('/geobase/list/street', $query);
}

sub get_nd($$$;$) {
	my ( $ville, $zip, $rue, $num ) = @_;
	my $query = "city=$ville&post_code=$zip&street=$rue";
	if (defined $num) {
		$query = $query."&street_nb=$num";
	}
	return get_reponse_get('/eligibility/search/nd', $query);
}

sub get_info_nd($$$) {
	my ( $nd, $inactive, $precab ) = @_;
	my $query = "nd=$nd";
	$query .= "&inactive=$inactive";
	$query .= "&precab=$precab";
	return get_reponse_get('/eligibility/info/nd', $query);
}

sub get_fiber($$$$) {
	my ( $ville, $rue, $num, $zip ) = @_;
	my $post = "{".
		"\"city\":\"$ville\",".
		"\"street\":\"$rue\",".
		"\"street_nb\":\"$num\",".
		"\"post_code\":\"$zip\"".
	"}";
	return get_reponse_post('/eligibility/send/fiber', $post);
}

sub get_sdsl($) {
	my ( $num ) = @_;
	my $post = "{\"phone_number\":\"$num\"}";
	return get_reponse_post('/eligibility/send/sdsl', $post);
}

sub get_adsl($$$) {
	my ( $num, $inactive, $precab ) = @_;
	my $post = "{".
		"\"phone_number\":\"$num\",".
		"\"inactive\":$inactive,".
		"\"precab\":$precab".
	"}";
	return get_reponse_post('/eligibility/send/adsl', $post);
}

sub get_contracts(;$) {
	my ( $state ) = @_;
	my $get = '';
	if ( defined($state) ) {
		$get = "last_state=$state";
	}
	return get_reponse_get('/contract/list', $get);
}

sub get_contract($) {
	my ( $ref ) = @_;
	my $get = "reference=$ref";
	return get_reponse_get('/contract/get', $get);
}

sub get_services() {
	return get_reponse_get('/service/list', '');
}

sub get_collecte() {
	return get_reponse_get('/service/list/collect', '');
}

my %product_code_par_debit = (
	"128k" => "160k.96k.dom",
	"512k" => "0.5m.160k",
	"512RE" => "0.5m.160k.re",
	"1M" => "1m.max",
	"2M" => "2m.max",
	"8M" => "10m.max",
	"18M" => "25m.max",
	"80M" => "100m.max"
);

# Usage interne: convertit une paire dbit/option en code Liazo
sub adsl_product_code($$) {
	my ($debit, $option) = @_;
	my $pcode = $product_code_par_debit{$debit};
	my $degroupage;
	my $dsl;
	if ($debit eq "80M") {
		$dsl = "vdsl";
	} else {
		$dsl = "adsl";
	}
	if ($option eq "LiazoOrangeNu") {
		$degroupage = "total";
	} else {
		$degroupage = "partial";
	}
	return "ft.cuivre.$dsl.$degroupage.monovc.$pcode.bpincluded";
}

# Commander une ligne, version complte
sub commande($$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$) {
	my (
	     $product,
	     $option,
	     $ndi,
	     $inactive,
	     $precab,

	     # contact commande
		$contact_name,
		$contact_phone_number, $contact_mobile_phone_number,
		$contact_email,

	     # install
		# contact sur place
		$install_phone_number,
		$install_post_code,
		$install_street_nb,
		$install_street_code, # optionnel
		$install_street, $install_street_complement,

		# optionnels
		$install_ensemble, $install_building,
		$install_stairs, $install_floor,
		$install_door,
		$install_logo,

		$install_city,
		$install_insee,

		$install_siret,
		$install_name_place,

		# Date de disponibilit des locaux
		$install_date_available,
		# Date dsire pour la mise  disposition du service
		$install_date_desired_available,

		# Contact pour l'installation
		$install_contact_name,
		$install_contact_phone_number, $install_contact_mobile_phone_number,
		$install_contact_email,

		# Rference du rendez-vous
		$install_rdv_reference,
		# Date de rendez-vous (format dd-mm-YYYY)
		$install_rdv_date,
		# Intervale du rendez-vous (8h-10h, 10h-12h, 13h-15h, 15h-17h)
		$install_rdv_range,

	    # client, optionnel, juste pour nous
		$client_reference, $client_comment,

		$test
	) = @_;

	my %params = ();
	$params{'product'} = $product;
	$params{'option'} = $option;

	$params{'ndi'} = $ndi;
	$params{'inactive'} = doJSONbool($inactive);
	$params{'precab'} = doJSONbool($precab);

	my %contact = ();
	$contact{'name'} = $contact_name;
	$contact{'phone_number'} = $contact_phone_number;
	$contact{'mobile_phone_number'} = $contact_mobile_phone_number;
	$contact{'email'} = $contact_email;
	$params{'contact'} = \%contact;

	my %install = ();
	$install{'phone_number'} = $install_phone_number;
	$install{'post_code'} = $install_post_code;
	$install{'street_nb'} = $install_street_nb;
	$install{'street_code'} = $install_street_code;
	$install{'street'} = $install_street;
	$install{'street_complement'} = $install_street_complement;
	$install{'ensemble'} = $install_ensemble;
	$install{'building'} = $install_building;
	$install{'stairs'} = $install_stairs;
	$install{'floor'} = $install_floor;
	$install{'door'} = $install_door;
	$install{'logo'} = $install_logo;
	$install{'city'} = $install_city;

	$install{'insee'} = $install_insee;
	$install{'siret'} = $install_siret;
	$install{'name_place'} = $install_name_place;
	$install{'date_available'} = $install_date_available;
	$install{'date_desired_available'} = $install_date_desired_available;
	$install{'contact_name'} = $install_contact_name;
	$install{'contact_phone_number'} = $install_contact_phone_number;
	$install{'contact_mobile_phone_number'} = $install_contact_mobile_phone_number;
	$install{'contact_email'} = $install_contact_email;

	# pour l'ADSL seulement
	$install{'rdv_reference'} = $install_rdv_reference;
	$install{'rdv_date'} = $install_rdv_date;
	$install{'rdv_range'} = $install_rdv_range;

	$params{'install'} = \%install;

	my %client = ();
	$client{'reference'} = $client_reference;
	$client{'comment'} = $client_comment;
	$params{'client'} = \%client;

	# pour le SDSL seulement
	#my %collect = ();
	#$collect{'reference'} = $collect_reference;
	#$collect{'vlan_range'} = $client_vlan_range;
	#$collect{'vlan'} = $client_vlan;
	#$params{'collect'} = %collect;

	my $post = encode_json(\%params);

	# resp. /order/send/sdsl
	__debug("commande $post");
	if ($test) {
		return "PR1234-1234";
	} else {
		my $result = get_reponse_post('/order/send/adsl', $post, 1);
		__debug("Liazo a rpondu");
		__debug("La reference obtenue est ".$result->{'reference'});
		return $result->{'reference'};
	}
}

# Commander une ligne, version ne demandant que les informations dont on dispose chez FDN
sub do_commande_adsl($$$$$$$$$$$$$$$$$$) {
	my (
	     $debit,
	     $option_degroupage,
	     $ndi,
	     $inactive,
	     $precab,

	     # install
		# contact sur place
		$install_name_place,
		$install_phone_number,
		$adresse,
		$install_post_code,
		$install_city,

		# Date de disponibilit des locaux
		$install_date_available,
		# Date dsire pour la mise  disposition du service
		$install_date_desired_available,

		# Rference du rendez-vous
		$install_rdv_reference,
		# Date de rendez-vous (format dd-mm-YYYY)
		$install_rdv_date,
		# Intervale du rendez-vous (8h-10h, 10h-12h, 13h-15h, 15h-17h)
		$install_rdv_range,

	    # client, optionnel, juste pour nous
		$lid,

		$validateur,

		$test
	) = @_;

	my $product = adsl_product_code($debit, $option_degroupage);
	my $options_liazo = [];
        my $suivi_tel = Config_Key($conf,'ariane','suivi_tel');
	if ($validateur) {
		my $tel = Config_Key('adsl/global.cfg',$validateur,'suivi_tel');
		if ($tel) {
			$suivi_tel = $tel;
		}
	}

	my ( $install_street_nb, $install_street );
	if ( defined($adresse) ) {
		$adresse =~ /^(\d*) *(.*)$/;
		($install_street_nb, $install_street) = ($1, $2);
	}

	return commande(
		$product,
		$options_liazo,
		$ndi,
		$inactive,
		$precab,

	     # contact commande, TODO
		'FDN',
		undef, undef,
		'buro@fdn.fr',

	     # install
		# contact sur place
		$install_phone_number,
		$install_post_code,
		$install_street_nb,
		undef, # street_code, optionnel
		$install_street, '',

		# prcisions adresse optionnelles
		undef, undef,
		undef, undef,
		undef,
		undef,

		$install_city,
		'',

		'419 897 855 00047',
		$install_name_place,

		# Date de disponibilit des locaux
		$install_date_available,
		# Date dsire pour la mise  disposition du service
		$install_date_desired_available,

		# Contact pour l'installation, TODO
		'FDN',
		$suivi_tel, undef,
		'suivi-adsl@fdn.fr',

		# Rference du rendez-vous
		$install_rdv_reference,
		# Date de rendez-vous (format dd-mm-YYYY)
		$install_rdv_date,
		# Intervale du rendez-vous (8h-10h, 10h-12h, 13h-15h, 15h-17h)
		$install_rdv_range,

	    # client, optionnel, juste pour nous
		$lid, undef,

		$test
	);
}
sub commande_adsl($$$$$$$$$$$$$$$$) {
	my (
	     $debit,
	     $option_degroupage,
	     $ndi,

	     # install
		# contact sur place
		$install_name_place,
		$install_phone_number,
		$adresse,
		$install_post_code,
		$install_city,

		# Date de disponibilit des locaux
		$install_date_available,
		# Date dsire pour la mise  disposition du service
		$install_date_desired_available,

		# Rference du rendez-vous
		$install_rdv_reference,
		# Date de rendez-vous (format dd-mm-YYYY)
		$install_rdv_date,
		# Intervale du rendez-vous (8h-10h, 10h-12h, 13h-15h, 15h-17h)
		$install_rdv_range,

	    # client, optionnel, juste pour nous
		$lid,

		$validateur,

		$test
	) = @_;

	my ( $inactive, $precab, $err, @info ) = get_eligibilite($ndi);
	if (defined $err) {
		debug("oops, erreur $err avant mme la commande\n");
		return undef;
	}

	do_commande_adsl(
	     $debit,
	     $option_degroupage,
	     $ndi,
	     $inactive,
	     $precab,

	     # install
		# contact sur place
		$install_name_place,
		$install_phone_number,
		$adresse,
		$install_post_code,
		$install_city,

		# Date de disponibilit des locaux
		$install_date_available,
		# Date dsire pour la mise  disposition du service
		$install_date_desired_available,

		# Rference du rendez-vous
		$install_rdv_reference,
		# Date de rendez-vous (format dd-mm-YYYY)
		$install_rdv_date,
		# Intervale du rendez-vous (8h-10h, 10h-12h, 13h-15h, 15h-17h)
		$install_rdv_range,

	    # client, optionnel, juste pour nous
		$lid,

		$validateur,

		$test
	);
}

# Rcuprer la liste des commandes
sub get_commandes($$) {
	my ( $start, $end ) = @_;
	my $get = "start=$start&end=$end";
	return get_reponse_get('/order/list', $get);
}

# Rcuprer les infos sur une commande
sub get_info_commande($) {
	my ( $ref ) = @_;
	my $get = "reference=$ref";
	return get_reponse_get('/order/get', $get);
}

# Appele de temps en temps par cron pour regarder o en sont les commandes
sub poll() {
	#__debug "commandes nouvelles";
	my $tmp = sql_transaction($dbh, 'select', 'select ORDRETD_ID from ORDRETD where ORDRETD_ETAT="new" and ORDRETD_LIGNE like "PR%"');
	my @list = FDN::Sql::Basic::__return_all($tmp);
	for my $res ( @list ) {
		my $oid = $res->{'ORDRETD_ID'};
		my $ordre = ordretd_par_id($oid);
		my $ref = $ordre->{'ORDRETD_LIGNE'};
		# pour tester avec une commande connue pour tre livre
		#$ref = "PR1609-1475";
		#__debug "commande nouvelle $oid rfrence liazo $ref";
		my $info = get_info_commande($ref);

		for my $status ( @{$info->{'status'}} ) {
			if ($status->{'status'} == 2) {
				# Commande confirme par Liazo
				__debug "commande $oid $ref valide";
				mark_sent( undef, ($ordre) );
				event_sent( ($ordre) );
			}
		}
	}

	#__debug "commandes envoyes";
	$tmp = sql_transaction($dbh, 'select', 'select ORDRETD_ID from ORDRETD where ORDRETD_ETAT="sent" and ORDRETD_LIGNE like "PR%"');
	@list = FDN::Sql::Basic::__return_all($tmp);
	for my $res ( @list ) {
		my $oid = $res->{'ORDRETD_ID'};
		my $ordre = ordretd_par_id($oid);
		my $ref = $ordre->{'ORDRETD_LIGNE'};
		# pour tester avec une commande connue pour tre livre
		#$ref = "PR1609-1475";
		#__debug "commande envoye $oid rfrence liazo $ref";
		my $info = get_info_commande($ref);

		for my $status ( @{$info->{'status'}} ) {
			if ($status->{'status'} == 6) {
				# Commande livre par Liazo
				__debug "commande $oid $ref livre";
				set_ordretd_etat($oid, 'ok');
				event_ok( ($ordre) );
			}
		}
	}
}

# Usage interne: rcuprer la base des urls de l'API Liazo
sub get_url() {
	my $protocol = Config_Key($conf,'ariane','protocol');
	my $url      = Config_Key($conf,'ariane','url');
	my $cgi      = Config_Key($conf,'ariane','cgi');
	$url = $protocol."://".$url.$cgi;
	return $url;
}

# Usage interne: rcuprer une rponse http et la dcoder.
sub get_reponse($) {
	my ( $request ) = @_;
	my $ua = LWP::UserAgent->new;
	$ua->timeout(60);
	my $response = $ua->request($request);
	if ( $response->message =~ /timeout/ ) {
		error "Le serveur Liazo n'a pas repondu assez vite.";
		return undef;
	}
	__debug "Reponse HTTP: ".$response->message;
	__debug "Reponse JSON ".$response->content;
	my $res;
	my $worked = eval '$res = decode_json($response->content);';
	if ( !defined($worked)) {
		error "Le serveur Liazo n'a pas rpondu au JSON: ".$@;
		return undef;
	}
	if ( $res->{'error'}{'code'} != 0 ) {
		error "Le serveur Liazo a rpondu ".$res->{'error'}{'msg'};
		return undef;
	}
	if ( !defined($res->{'result'}) ) {
		error "Le serveur Liazo n'a rpondu de section result ";
		return undef;
	}
	return $res->{'result'};
}

# Poster une requte POST  Liazo
sub get_reponse_post($$;$) {
	my ( $cgi, $json, $dosign ) = @_;
	my $url = get_url().$cgi;
	__debug "URL complete: $url";
	__debug "JSON complete: $json";
	my $header = new HTTP::Headers;
	my $key    = Config_Key($conf,'ariane','api_key');
	$header->header('Content-Type' => 'text/plain; charset=utf-8');
	$header->header('X-LAPI-KEY' => $key);
	if ($dosign) {
		my $sign   = sign($json);
		$header->header('X-LAPI-SIGN' => $sign);
	}
	my $request = new HTTP::Request ('POST', $url, $header );
	$request->content($json);
	return get_reponse($request);
}

# Poster une requte GET  Liazo
sub get_reponse_get($) {
	my ( $cgi, $get ) = @_;
	my $url = get_url().$cgi;
	$url = $url.'?'.$get;
	__debug "URL complete: $url";
	my $header = new HTTP::Headers;
	my $key    = Config_Key($conf,'ariane','api_key');
	$header->header('X-LAPI-KEY' => $key);
	my $request = new HTTP::Request ('GET', $url, $header );
	return get_reponse($request);
}

1;
