#!/usr/bin/perl

use strict;
use CGI;
use FDN::Common::Config;
use FDN::Web::Forms;
use FDN::Web::Array;
use FDN::Adsl::Utils;
use FDN::Adsl::Basic;
use FDN::Adsl::Infos;
use FDN::Sql::Workflow::Run;
use FDN::Sql::Workflow::Basic;

my $cgi = new CGI;
my $vars = $cgi->Vars;

workflow_select_db('adsl');

if ( ! defined $vars->{'ligneid'} ) {
	my @lignes = ligne_par_state('new');
	my $page = tableau_2d('adsl/rewrite.tbl','ligne-fas',\@lignes);

	my @tasks_liazo = task_par_name_state('change-adslL', 'running');
	my @change_lignes;
        foreach my $task ( @tasks_liazo ) {
		my $props = get_task_properties($task->{'TASK_ID'});
		my $ligne = ligne_par_id($props->{'ligneid'});
		my $did = $props->{'deplid'};
		$ligne->{'DEPLOIEMENT_ID'} = $did;
		push @change_lignes, $ligne;
	}

	$page .= tableau_2d('adsl/rewrite.tbl','ligne-change',\@change_lignes);

	eject_page($page);
	exit(0);
}

my $lid = $vars->{'ligneid'};
my $dpl;
my $did = $vars->{'deplid'};
if (defined($did)) {
	$dpl = deploiement_par_id($did);
} else {
	$dpl = deploiement_par_ligne_id($lid);
	$did = $dpl->{'DEPLOIEMENT_ID'};
}
my $tid = $dpl->{'TASK_ID'};
my $props = get_task_properties($tid);
my $eid = $props->{'eligibilite-id'};
my $elig = eligibilite_par_id($eid);
if ( $vars->{'do'} ne 'yes' ) {
	my $ligne = ligne_par_id($lid);
	my $cid = $ligne->{'CLIENT_ID'};
	my $client = client_par_id($cid);
	my $adresse = adresse_par_id($ligne->{'ADRESSE_ID'});
	my $page;
	my @infos;

	$page  = tableau_1d('adsl/rewrite.tbl','ligne',$ligne,"Description de la ligne");
	if ($elig) {
		$page .= "<p>Retour ligibilit: ".$elig->{'ELIGIBILITE_REPONSE'}."</p>";
	}
	$page .= tableau_1d('adsl/rewrite.tbl','adresse',$adresse,"Adresse de la ligne");
	$page .= tableau_1d('adsl/rewrite.tbl','client',$client,"Description du client");
	if ( $client->{'PRELEVEMT_ID'} ) {
		my $prel = prelevemt_par_id($client->{'PRELEVEMT_ID'});
		$page .= tableau_1d('adsl/rewrite.tbl','prelevemt',$prel,"Compte bancaire pour prelevement");
	} else {
		$page .= '<div class="error">Pas de compte bancaire pour les prelevements. Paiement par facture ?</div>'."\n\n";
	}

	@infos = toutes_infos_ligne($lid);
	$page .= tableau_2d('adsl/rewrite.tbl','infos',\@infos,"Informations sur la ligne");

	@infos = toutes_infos_client($cid);
	$page .= tableau_2d('adsl/rewrite.tbl','infos',\@infos,"Informations sur le client");

	if ($ligne->{'LIGNE_STATE'} eq 'new') {
		$page .= "<p>Il faut vrifier la prsence de tous ces lments:<br>\n".
			 "<b>Nouvel adhrent:</b></p>\n".
			 "<ul>\n".
			 "<li>l'image du bulletin d'adhsion;</li>\n".
			 "<li>l'image de l'autorisation de prlvement;</li>\n".
			 "<li>la saisie du RIB pour le prlvement (l'item &nbsp;Paiment&nbsp;\n".
			 "    dans la description du client ne doit pas mentionner de Facture, mais\n".
			 "    l'identifiant du RIB);</li>\n".
			 "<li>l'image de la pice d'identit de l'adhrent;</li>\n".
			 "<li>l'image du justificatif (carte d'tudiant, etc) si l'adhrent\n".
			 "    demande le tarif prfrentiel.\n".
			 "</ul>\n".
			 "<p><b>Nouvelle ligne ADSL:</b></p>\n".
			 "<ul>\n".
			 "<li>l'image du bulletin d'abonnement pour la ligne (deux pages, donc\n".
			 "    deux images)</li>\n".
			 "<li>l'image de la facture France Tlcom</li>\n".
			 "<li>ou l'image d'un justificatif de domicile correspondant  l'adresse selon l'ligibilit</li>\n".
			 #"<li>l'image du mandat de dgroupage, si le dbit souscrit est en\n".
			 #"    dgroup</li>\n".
			 "</ul>\n".
			 "<p>Si un des lments manque, il faut retourner le scanner, ou attendre\n".
			 "de le recevoir selon le cas. <b>Ne pas valider de ligne s'il manque des\n".
			 "documents!</b></p>";
		$page .= "<p>La construction sera demande pour le ".$props->{'date'}.". Cette date peut tre change dans <a href=\"modif-taskprop.cgi?taskid=$tid\">la tche</a><br>\n";
		$page .= '<p><a href="paye-fas.cgi?ligneid='.$lid.'&do=yes">Valider</a></p>';
	} else {
		$page .= "<p>Il faut lancer la rsiliation de la ligne chez Nerim pour le ".$props->{'date'}." avant de valider ci-dessous<br>\n";
		$page .= "<p>Cette date peut tre change dans <a href=\"modif-taskprop.cgi?taskid=$tid\">la tche</a><br>\n";
		$page .= '<p><a href="paye-fas.cgi?ligneid='.$lid.'&deplid='.$did.'&do=yes">Valider</a></p>';
	}
	eject_page($page);
	exit(0);
}

# Noter qui a valid la commande
my $newprop = { 'validateur' => $cgi->remote_user() };
set_task_properties($tid, $newprop);

my $ligne = ligne_par_id($lid);
my $tel = $ligne->{'LIGNE_TEL'};
my $suff = "";
if ($ligne->{'LIGNE_STATE'} ne 'new') {
	$suff = "-".$did;
}
my $event = 'FAS-'.$tel.$suff;

# Mise en facturation sera faite en fin de construction.
#set_ligne_dtfac($lid);
send_event($event);
#TODO Creer une creance negative correspondant au paiement.
my $index = Config_Key('adsl/global.cfg','web','index');
print "Location: $index\n\n";

